//#define DUMP

#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoInfo.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/SoLists.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/actions/SoWriteAction.h>
#include "SceneParser.h"
#include "SoURLCompressedTexture2.h"
#include "SoURLTexture2.h"
#include "VRML.h"
#include "Global.h"
#include "SoXtWebViewer.h"
#include <iostream.h>
#include <Xm/PushBG.h>
#include "DataProvider.h"

extern SoXtWebViewer *Viewer;
extern DataProvider *dp;


void 
changeTextures()
{
  SoNodeList *textureNodeList = new SoNodeList;

  SoNode::getByName("PGM_Texture", *textureNodeList);
 
  for (int i = 0 ; i < textureNodeList->getLength() ; i++ )
    {
      SoGroup *parent = (SoGroup*)((*textureNodeList)[i]);

      SoInfo *si = (SoInfo*)(parent->getChild(0));


      char buf[256];      
      strcpy(buf, (char*)si->string.getValue().getString());
      int len = strlen(buf);
      char ext[4];
      ext[0] = buf[len-3];
      ext[1] = buf[len-2];
      ext[2] = buf[len-1];
      ext[3] = 0;

      SoTexture2 *tp;
      
      Timer *tt = new Timer();
      
      if (!strcmp(ext, "twv") || !strcmp(ext, "cps")) 
	tp = new 
	  SoURLCompressedTexture2((char*)si->string.getValue().getString());
      if (!strcmp(ext, "ppm")) 
	tp = new 
	  SoURLTexture2( (char*)si->string.getValue().getString());
      tt->stop("texture created");
      

      ((SoGroup*)parent)->replaceChild(si,tp);
    }
  delete textureNodeList;
}



void 
changeTerrains()
{
  SoNodeList *terrainNodeList = new SoNodeList;

  SoNode::getByName("PGM_Terrain", *terrainNodeList);
 
  for (int i = 0 ; i < terrainNodeList->getLength() ; i++ )
    {
      SoGroup *parent = (SoGroup*)((*terrainNodeList)[i]);

      SoInfo *si = (SoInfo*)parent->getChild(0);

      char* str = (char*)si->string.getValue().getString();

      char* p = strchr(str,'*');
      char* meshTreshStr = p+1;
      *p = '\0';
      float meshTresh = atof(meshTreshStr);

      SoSeparator *urlTerrain = getURLTerrain(str,meshTresh);

      ((SoGroup*)parent)->replaceChild(si,urlTerrain);
    }
  delete terrainNodeList;
}

//button stuff

void
switchToNextChild(char *switchName)
{
  SoSwitch *node;
  SoNodeList *list = new SoNodeList;
  SoNode::getByName(switchName, *list);
  int len = list->getLength();
  for(int i=0; i<len; i++){
    node = (SoSwitch*) (*list)[i];
    long currentChild = node->whichChild.getValue();
    currentChild++;
    currentChild = currentChild % node->getNumChildren();
    node->whichChild.setValue((int32_t)currentChild);
    node = (SoSwitch*)SoNode::getByName(switchName);
  }
  delete list;
}

void 
pushButtonCB(Widget, int data, void *)
{
  char buf[256];
  
  sprintf(buf, "Option_%d", data);
  switchToNextChild(buf);
}

int usingButton[6] = {0, 0, 0, 0, 0, 0};
Widget buttons[6], dumbParent;
char buttonCaption[6][3] = { "A", "B", "C", "D", "E", "F" };

Arg args[20];
int argc = 0;

void 
addButtons(SoSeparator *root)
{
  int i;
  char buf[256];
  
  for(i=0; i<6; i++)
    if(usingButton[i]) Viewer->removeAppPushButton(buttons[i]);
  
  dumbParent = Viewer->getAppPushButtonParent();

  for(i=0; i<6; i++){
    buf[0] = 0;
    sprintf(buf, "Option_%d", i);
    usingButton[i] = (SoNode::getByName(buf))? TRUE: FALSE;
  }
  
  XtSetArg(args[argc], XmNshadowThickness, 3);
  argc++;
  XtSetArg(args[argc], XmNhighlightThickness, 0);
  argc++;
  
  for(i=0; i<6; i++)
    if(usingButton[i]) {
      buttons[i] = XmCreatePushButtonGadget(dumbParent, 
					   buttonCaption[i], 
					   args, argc);
      XtAddCallback(buttons[i], 
		    XmNactivateCallback,
		    (XtCallbackProc) pushButtonCB,
		    (XtPointer) i); 
      Viewer->addAppPushButton(buttons[i]);
    }
};


void ParseSceneGraph( SoSeparator *root )
{
  //  info("Changing the textures...");
  changeTextures();

  //  info("Changing the DEM...");
  changeTerrains();

  //  info("Updating the toolbar...");
  //  addButtons(root);
  //  info("...done\n");

#ifdef DUMP
  info("Dumping the scene...");
  SoWriteAction myAction;
  myAction.getOutput()->openFile("dump.iv");
  myAction.getOutput()->setBinary(FALSE);
  myAction.apply(root);
  myAction.getOutput()->closeFile();
  info("...done\n");
#endif //DUMP
  
}

